package com.ashokit.collections.dao;

import java.util.Date;
import java.util.List;
import java.util.concurrent.CompletableFuture;

import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.CrudRepository;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Repository;

import com.ashokit.collections.model.CustomType;
import com.ashokit.collections.model.Person;

@Repository
public interface PersonDao extends JpaRepository<Person, Integer> {
	//SaveAll and FindAllById already available in CRUDRepository itself 
	//so we no need to write the code
	
	//Derived Queries
	
	public Person findByLastNameAndFirstName(String lastName,String firstName);
	//select * from tbl_person where last_name=lastName and first_name=lastName
	//The above query should result only one record ==> If it retrieves more than one record it will fail
	//Order is imporant and the same order database will take inputs
	
	
	public Iterable<Person> findByLastNameOrFirstName(String lastName,String firstName);
	//select * from tbl_person where last_name=lastName or first_name=firstName
	
	
	public List<Person> findByLastNameOrderByCreatedDateDesc(String lastName);
	//select * from tbl_person where last_name=lastName order by created_date desc;

	
	public List<Person> findByAgeLessThanEqual(Integer age);
	//select * from tbl_person where age<=age;
	
	
	public List<Person> findByFirstNameLike(String firstName);
	//select * from tbl_person where first_name like '%firstname%';
	
	public List<Person> findByLastNameAndAgeLessThanEqual(String lastName,int age);
	//select * from tbl_person where last_name=lastName and age<=age;
	
	public List<Person> findByCreatedDateBetween(Date startdate,Date endDate);
	//select * from tbl_person where created_date>=startdate and created_date<=endDate
	
	//NamedQuery
	public List<Person> getDataByLastName(String lastName); //Abstract Method
	//WE will proviede the implmentation Model Layer 
		
		
	
	//NamedQuery
	public List<CustomType> getFewcolumns(String lastName); //Abstract Method
	//WE will proviede the implmentation Model Layer 
	
	
	/* 
	 * Both Abstract Method and Implementation in Interface Layer itself ==>
	 * Query and NativeQuery
	 */
	
	
	
	@Query(value="SELECT p FROM Person p where p.firstName=?1 OR p.email=?2") //Implementation
	List<Person> findPersonInfobyFirstNameorEmail(String firstName,String email);//Abstract method


	@Query(value="SELECT * FROM tbl_person p where p.first_name=?1",nativeQuery = true)
	List<Person> findPersonINfobyFirstname(String firstName);

	
	
	//Pagination Logic
	

	public List<Person> findByLastName(String lastName, Pageable pageable);
	
	
	
	//Synch Code
	public List<Person> findByEmail(String email); //Synch Method (Default is Synchronous)
	
	
	//Asynchronous Code
	@Async
	public CompletableFuture<Person> findByemail(String email); //This is Asynchronous

		

}
